package com.flyfish.framework.domain.base;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.flyfish.framework.builder.CriteriaBuilder;
import com.flyfish.framework.domain.po.User;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.mongodb.core.query.Criteria;

import java.lang.reflect.ParameterizedType;
import java.util.List;

/**
 * 基本的查询实体
 *
 * @author Mr.Wang
 */
public class BaseQo<T extends Domain> implements Qo<T> {

    protected Pageable pageable;

    protected List<T> result;

    protected User user;

    public Qo<T> accept(List<T> result, Pageable pageable) {
        this.pageable = pageable;
        this.result = result;
        return this;
    }

    public Qo<T> accept(Pageable pageable) {
        this.pageable = pageable;
        return this;
    }

    @Override
    public Pageable getPageable() {
        return pageable;
    }

    @JsonIgnore
    @Override
    public void setPageable(Pageable pageable) {
        this.pageable = pageable;
    }

    /**
     * 获取当前用户
     *
     * @return 结果
     */
    @Override
    public User getUser() {
        return user;
    }

    /**
     * 当前用户
     *
     * @param user 用户数据
     */
    @Override
    public void setUser(User user) {
        this.user = user;
    }

    @Override
    public List<T> getResult() {
        return result;
    }

    /**
     * 设置结果集，用于单例部署时快速封装
     *
     * @param result 查询结果集
     */
    @JsonIgnore
    @Override
    public void setResult(List<T> result) {
        this.result = result;
    }

    /**
     * 定义查询规则的方法
     *
     * @return 包含匹配规则的example
     */
    @JsonIgnore
    @Override
    public Example<T> getExample() {
        return null;
    }

    /**
     * 获取data-mongo的对象Criteria
     *
     * @return 结果
     */
    @Override
    public Criteria getCriteria() {
        CriteriaBuilder<T> criteriaBuilder = criteriaBuilder();
        if (null != criteriaBuilder) {
            return criteriaBuilder.build();
        }
        return null;
    }

    /**
     * 内置的builder，重写后自动合并结果
     *
     * @return 结果
     */
    public CriteriaBuilder<T> criteriaBuilder() {
        return null;
    }

    /**
     * 让值全部包含在Pojo里
     *
     * @return 结果
     */
    @Override
    @SuppressWarnings("unchecked")
    public Class<T> pojoType() {
        ParameterizedType type = (ParameterizedType) this.getClass().getGenericSuperclass();
        return (Class<T>) type.getActualTypeArguments()[0];
    }


    @Override
    public Sort sorts() {
        return Sort.by(Sort.Order.desc("createTime"));
    }
}
