package com.flyfish.framework.handler;

import com.flyfish.framework.bean.Result;
import com.flyfish.framework.configuration.jwt.TokenProvider;
import com.flyfish.framework.service.AuthenticationAuditor;
import lombok.RequiredArgsConstructor;
import org.springframework.http.MediaType;
import org.springframework.http.server.reactive.ServerHttpResponse;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.web.server.WebFilterExchange;
import org.springframework.security.web.server.authentication.logout.ServerLogoutSuccessHandler;
import reactor.core.publisher.Mono;

/**
 * 退出登录的回调
 *
 * @author wangyu
 */
@RequiredArgsConstructor
public class JsonLogoutSuccessHandler implements ServerLogoutSuccessHandler {

    // 数据块工厂
    private final AuthenticationAuditor authenticationAuditor;

    // token提供者
    private final TokenProvider tokenProvider;

    @Override
    public Mono<Void> onLogoutSuccess(WebFilterExchange webFilterExchange, Authentication authentication) {
        ServerHttpResponse response = webFilterExchange.getExchange().getResponse();
        response.getHeaders().setContentType(MediaType.APPLICATION_JSON);
        tokenProvider.removeToken(webFilterExchange.getExchange());
        Object principal = authentication.getPrincipal();
        if ("anonymous".equals(principal)) {
            return response.writeWith(authenticationAuditor.transform(Result.ok()));
        }
        // 记录日志
        return authenticationAuditor.logout((UserDetails) authentication.getPrincipal())
                .then(response.writeWith(authenticationAuditor.transform(Result.ok())));
    }
}
